@@ -1,19 +1,18 @@
 import copy
 import warnings
-from textwrap import dedent
 import collections
 import tempfile
 import sys
 import os
 import sqlite3
-import constants
-import version
-import parser
-import bins
-import helpers
-import feature
-import interface
-import iterators
+import six
+from gffutils import constants
+from gffutils import version
+from gffutils import bins
+from gffutils import helpers
+from gffutils import feature
+from gffutils import interface
+from gffutils import iterators
 
 import logging
 
@@ -32,7 +31,8 @@ class _DBCreator(object):
                  default_encoding='utf-8',
                  infer_gene_extent=True,
                  force_merge_fields=None,
-                 text_factory=None):
+                 text_factory=sqlite3.OptimizedUnicode,
+                 pragmas=constants.default_pragmas):
         """
         Base class for _GFFDBCreator and _GTFDBCreator; see create_db()
         function for docs
@@ -51,6 +51,7 @@ class _DBCreator(object):
                     "this may result in unusable features." % w)
 
         self.force_merge_fields = force_merge_fields
+        self.pragmas = pragmas
         self.merge_strategy = merge_strategy
         self.default_encoding = default_encoding
         self.infer_gene_extent = infer_gene_extent
@@ -60,7 +61,7 @@ class _DBCreator(object):
                 os.unlink(dbfn)
         self.dbfn = dbfn
         self.id_spec = id_spec
-        if isinstance(dbfn, basestring):
+        if isinstance(dbfn, six.string_types):
             conn = sqlite3.connect(dbfn)
         else:
             conn = dbfn
@@ -103,7 +104,7 @@ class _DBCreator(object):
         """
 
         # If id_spec is a string, convert to iterable for later
-        if isinstance(self.id_spec, basestring):
+        if isinstance(self.id_spec, six.string_types):
             id_key = [self.id_spec]
 
         elif hasattr(self.id_spec, '__call__'):
@@ -114,7 +115,7 @@ class _DBCreator(object):
         elif isinstance(self.id_spec, dict):
             try:
                 id_key = self.id_spec[f.featuretype]
-                if isinstance(id_key, basestring):
+                if isinstance(id_key, six.string_types):
                     id_key = [id_key]
 
             # Otherwise, use default auto-increment.
@@ -285,7 +286,7 @@ class _DBCreator(object):
 
                 # Set the final merged non-attributes
                 for k, v in final_fields.items():
-                    setattr(existing_feature, k, ','.join(map(str, v)))
+                    setattr(existing_feature, k, ','.join(sorted(map(str, v))))
 
                 if self.verbose == 'debug':
                     logger.debug('\nMERGED:\n%s' % existing_feature)
@@ -363,27 +364,32 @@ class _DBCreator(object):
             c.execute("DROP INDEX IF EXISTS ?", (index,))
         self.conn.commit()
 
+    def set_pragmas(self, pragmas):
+        """
+        Set pragmas for the current database connection.
+
+        Parameters
+        ----------
+        pragmas : dict
+            Dictionary of pragmas; see constants.default_pragmas for a template
+            and http://www.sqlite.org/pragma.html for a full list.
+        """
+        self.pragmas = pragmas
+        c = self.conn.cursor()
+        c.executescript(
+            ';\n'.join(
+                ['PRAGMA %s=%s' % i for i in self.pragmas.items()]
+            )
+        )
+        self.conn.commit()
+
     def _init_tables(self):
         """
         Table creation
         """
         c = self.conn.cursor()
         v = sqlite3.sqlite_version_info
-
-        # WAL is in sqlite 3.7+.
-        if v[0] >= 3 and v[1] >= 7:
-            c.executescript(
-                '''
-                PRAGMA synchronous=NORMAL;
-                PRAGMA journal_mode=WAL;
-                ''')
-
-        c.executescript(
-            '''
-            PRAGMA main.page_size=4096;
-            PRAGMA main.cache_size=10000;
-            ''')
-
+        self.set_pragmas(self.pragmas)
         c.executescript(constants.SCHEMA)
         self.conn.commit()
 
@@ -409,7 +415,7 @@ class _DBCreator(object):
         c.executemany(
             '''
             INSERT OR REPLACE INTO autoincrements VALUES (?, ?)
-            ''', self._autoincrements.items())
+            ''', list(self._autoincrements.items()))
 
         # These indexes are *well* worth the effort and extra storage: over
         # 500x speedup on code like this:
@@ -454,8 +460,8 @@ class _DBCreator(object):
         Execute a query directly on the database.
         """
         c = self.conn.cursor()
-        c.execute(query)
-        for i in cursor:
+        result = c.execute(query)
+        for i in result:
             yield i
 
     def _insert(self, feature, cursor):
@@ -491,14 +497,17 @@ class _GFFDBCreator(_DBCreator):
         # Compared to a benchmark of doing each insert separately:
         # executemany using a list of dicts to iterate over is ~15% slower
         # executemany using a list of tuples to iterate over is ~8% faster
-
+        features_seen = None
         _features, _relations = [], []
         for i, f in enumerate(lines):
+            features_seen = i
 
             # Percent complete
+
             if self.verbose:
                 if i % 1000 == 0:
-                    logger.info(msg % i)
+                    sys.stderr.write(msg % i)
+                    sys.stderr.flush()
 
             # TODO: handle ID creation here...should be combined with the
             # INSERT below (that is, don't IGNORE below but catch the error and
@@ -542,6 +551,8 @@ class _GFFDBCreator(_DBCreator):
                         INSERT OR IGNORE INTO relations VALUES
                         (?, ?, 1)
                         ''', (parent, f.id))
+        if features_seen is None:
+            raise ValueError("No lines parsed -- was an empty file provided?")
 
         self.conn.commit()
         if self.verbose:
@@ -617,7 +628,9 @@ class _GTFDBCreator(_DBCreator):
         c = self.conn.cursor()
 
         last_perc = 0
+        lines_seen = None
         for i, f in enumerate(lines):
+            lines_seen = i
 
             # Percent complete
             if self.verbose:
@@ -686,10 +699,12 @@ class _GTFDBCreator(_DBCreator):
                 ''', relations
             )
 
+        if lines_seen is None:
+            raise ValueError("No lines parsed -- was an empty file provided?")
         logger.info('Committing changes')
         self.conn.commit()
         if self.verbose:
-            sys.stderr.write((msg % i) + '\n')
+            logger.info(msg % i)
 
     def _update_relations(self):
 
@@ -834,7 +849,7 @@ class _GTFDBCreator(_DBCreator):
             keys = ['parent', 'seqid', 'start', 'end', 'strand',
                     'featuretype', 'bin', 'attributes']
             for line in open(fout.name):
-                d = dict(zip(keys, line.strip().split('\t')))
+                d = dict(list(zip(keys, line.strip().split('\t'))))
                 d.pop('parent')
                 d['score'] = '.'
                 d['source'] = 'gffutils_derived'
@@ -882,8 +897,9 @@ def create_db(data, dbfn, id_spec=None, force=False, verbose=False,
               gtf_transcript_key='transcript_id', gtf_gene_key='gene_id',
               gtf_subfeature='exon', force_gff=False,
               force_dialect_check=False, from_string=False, keep_order=False,
-              text_factory=None, infer_gene_extent=True,
-              force_merge_fields=None):
+              text_factory=sqlite3.OptimizedUnicode, infer_gene_extent=True,
+              force_merge_fields=None, pragmas=constants.default_pragmas,
+              sort_attribute_values=False):
     """
     Create a database from a GFF or GTF file.
 
@@ -1035,6 +1051,23 @@ def create_db(data, dbfn, id_spec=None, force=False, verbose=False,
         strings of comma-separated values.  Note that 'start' and 'end' are not
         available, since these fields need to be integers.
 
+    text_factory : callable
+        Text factory to use for the sqlite3 database.  See
+        https://docs.python.org/2/library/\
+                sqlite3.html#sqlite3.Connection.text_factory
+        for details. The default sqlite3.OptimizedUnicode will return Unicode
+        objects only for non-ASCII data, and bytestrings otherwise.
+
+    pragmas : dict
+        Dictionary of pragmas used when creating the sqlite3 database. See
+        http://www.sqlite.org/pragma.html for a list of available pragmas.  The
+        defaults are stored in constants.default_pragmas, which can be used as
+        a template for supplying a custom dictionary.
+
+    sort_attribute_values : bool
+        All features returned from the database will have their attribute
+        values sorted.  Typically this is only useful for testing, since this
+        can get time-consuming for large numbers of features.
     """
     kwargs = dict(
         data=data, checklines=checklines, transform=transform,
@@ -1078,12 +1111,17 @@ def create_db(data, dbfn, id_spec=None, force=False, verbose=False,
     c = cls(dbfn=dbfn, id_spec=id_spec, force=force, verbose=verbose,
             merge_strategy=merge_strategy, text_factory=text_factory,
             infer_gene_extent=infer_gene_extent,
-            force_merge_fields=force_merge_fields, **kwargs)
+            force_merge_fields=force_merge_fields, pragmas=pragmas, **kwargs)
 
     c.create()
     if dbfn == ':memory:':
-        db = interface.FeatureDB(c.conn, keep_order=keep_order)
+        db = interface.FeatureDB(c.conn, keep_order=keep_order,
+                                 pragmas=pragmas,
+                                 sort_attribute_values=sort_attribute_values,
+                                 text_factory=text_factory)
     else:
-        db = interface.FeatureDB(c, keep_order=keep_order)
+        db = interface.FeatureDB(c, keep_order=keep_order, pragmas=pragmas,
+                                 sort_attribute_values=sort_attribute_values,
+                                 text_factory=text_factory)
 
     return db
