@@ -1,69 +1,12 @@
-import collections
+import six
 import simplejson
-import helpers
-import constants
-import parser
-import bins
+from gffutils import constants
+from gffutils import helpers
+from gffutils import parser
+from gffutils import bins
+from gffutils.attributes import dict_class
 
 
-# http://stackoverflow.com/a/3387975
-class Attributes(collections.MutableMapping):
-    '''
-    Mostly like a dictionary, but requires values to be lists
-    '''
-    def __init__(self, *args, **kwargs):
-        self._d = dict()
-        self.update(*args, **kwargs)
-
-    def __setitem__(self, key, value):
-        if not isinstance(value, list):
-            raise ValueError(
-                'Attribute value "%s" for key "%s" must be a list'
-                % (repr(value), repr(key)))
-        self._d[key] = value
-
-    def __getitem__(self, key):
-        return self._d[key]
-
-    def __delitem__(self, key):
-        del self._d[key]
-
-    def __iter__(self):
-        return iter(self.keys)
-
-    def __len__(self):
-        return len(self._d)
-
-    def keys(self):
-        return self._d.keys()
-
-    def values(self):
-        return self._d.values()
-
-    def items(self):
-        return self._d.items()
-
-    def __str__(self):
-        s = []
-        for i in self.items():
-            s.append("%s: %s" % i)
-        return '\n'.join(s)
-
-    def update(self, *args, **kwargs):
-        for k, v in dict(*args, **kwargs).iteritems():
-            self[k] = v
-
-
-# Useful for profiling: which dictionary-like class to store attributes in.
-# This is used in Feature below and in parser.py
-
-#dict_class = Attributes
-dict_class = dict
-#dict_class = helper_classes.DefaultOrderedDict
-#dict_class = collections.defaultdict
-#dict_class = collections.OrderedDict
-#dict_class = helper_classes.DefaultListOrderedDict
-
 _position_lookup = dict(enumerate(['seqid', 'source', 'featuretype', 'start',
                                    'end', 'score', 'strand', 'frame',
                                    'attributes']))
@@ -73,7 +16,8 @@ class Feature(object):
     def __init__(self, seqid=".", source=".", featuretype=".",
                  start=".", end=".", score=".", strand=".", frame=".",
                  attributes=None, extra=None, bin=None, id=None, dialect=None,
-                 file_order=None, keep_order=False):
+                 file_order=None, keep_order=False,
+                 sort_attribute_values=False):
         """
         Represents a feature from the database.
 
@@ -158,6 +102,12 @@ class Feature(object):
             order specified in the dialect.  Disabled by default, since this
             sorting step is time-consuming over many features.
 
+        sort_attribute_values : bool
+            If True, then the values of each attribute will be sorted when the
+            feature is printed.  Mostly useful for testing, where the order is
+            important for checking against expected values. Disabled by
+            default, since it can be time-consuming over many features.
+
         """
         # start/end can be provided as int-like, ".", or None, but will be
         # converted to int or None
@@ -179,7 +129,7 @@ class Feature(object):
         # for testing.
         attributes = attributes or dict_class()
 
-        if isinstance(attributes, basestring):
+        if isinstance(attributes, six.string_types):
             try:
                 attributes = helpers._unjsonify(attributes, isattributes=True)
 
@@ -195,19 +145,12 @@ class Feature(object):
         # If string, then try un-JSONifying it into a list; if that doesn't
         # work then assume it's tab-delimited and convert to a list.
         extra = extra or []
-        if isinstance(extra, basestring):
+        if isinstance(extra, six.string_types):
             try:
                 extra = helpers._unjsonify(extra)
             except simplejson.JSONDecodeError:
                 extra = extra.split('\t')
 
-        # Calculate bin if not provided
-        if bin is None:
-            try:
-                bin = bins.bins(start, end, one=True)
-            except TypeError:
-                bin = None
-
         self.seqid = seqid
         self.source = source
         self.featuretype = featuretype
@@ -218,11 +161,20 @@ class Feature(object):
         self.frame = frame
         self.attributes = attributes
         self.extra = extra
-        self.bin = bin
+        self.bin = self.calc_bin(bin)
         self.id = id
         self.dialect = dialect or constants.dialect
         self.file_order = file_order
         self.keep_order = keep_order
+        self.sort_attribute_values = sort_attribute_values
+
+    def calc_bin(self, _bin=None):
+        if _bin is None:
+            try:
+                _bin = bins.bins(self.start, self.end, one=True)
+            except TypeError:
+                _bin = None
+        return _bin
 
     def __repr__(self):
         memory_loc = hex(id(self))
@@ -260,7 +212,10 @@ class Feature(object):
             self.attributes[key] = value
 
     def __str__(self):
-        return unicode(self).encode('utf-8')
+        if six.PY3:
+            return self.__unicode__()
+        else:
+            return unicode(self).encode('utf-8')
 
     def __unicode__(self):
         # All fields but attributes (and extra).
@@ -279,7 +234,8 @@ class Feature(object):
 
         # Reconstruct from dict and dialect
         reconstructed_attributes = parser._reconstruct(
-            self.attributes, self.dialect, keep_order=self.keep_order)
+            self.attributes, self.dialect, keep_order=self.keep_order,
+            sort_attribute_values=self.sort_attribute_values)
 
         # Final line includes reconstructed as well as any previously-added
         # "extra" fields
@@ -332,7 +288,7 @@ class Feature(object):
                 self.id, self.seqid, self.source, self.featuretype, self.start,
                 self.end, self.score, self.strand, self.frame,
                 helpers._jsonify(self.attributes),
-                helpers._jsonify(self.extra), self.bin
+                helpers._jsonify(self.extra), self.calc_bin()
             )
         return (
             self.id.decode(encoding), self.seqid.decode(encoding),
@@ -340,7 +296,7 @@ class Feature(object):
             self.start, self.end, self.score.decode(encoding),
             self.strand.decode(encoding), self.frame.decode(encoding),
             helpers._jsonify(self.attributes).decode(encoding),
-            helpers._jsonify(self.extra).decode(encoding), self.bin
+            helpers._jsonify(self.extra).decode(encoding), self.calc_bin()
         )
 
 
@@ -385,7 +341,7 @@ def feature_from_line(line, dialect=None, strict=True, keep_order=False):
     except IndexError:
         attr_string = ""
     attrs, _dialect = parser._split_keyvals(attr_string, dialect=dialect)
-    d = dict(zip(constants._gffkeys, fields))
+    d = dict(list(zip(constants._gffkeys, fields)))
     d['attributes'] = attrs
     d['extra'] = fields[9:]
     d['keep_order'] = keep_order
