@@ -1,12 +1,12 @@
+import os
+import six
 import sqlite3
 import shutil
-import tempfile
-
-import create
-import bins
-import helpers
-import constants
-from feature import Feature, feature_from_line
+from gffutils import bins
+from gffutils import helpers
+from gffutils import constants
+from gffutils.feature import Feature
+from gffutils.exceptions import FeatureNotFoundError
 
 
 class FeatureDB(object):
@@ -39,8 +39,10 @@ class FeatureDB(object):
             within `limit`. Only relevant when `limit` is not None.
     """
 
-    def __init__(self, dbfn, text_factory=None, default_encoding='utf-8',
-                 keep_order=False):
+    def __init__(self, dbfn, default_encoding='utf-8', keep_order=False,
+                 pragmas=constants.default_pragmas,
+                 sort_attribute_values=False,
+                 text_factory=sqlite3.OptimizedUnicode):
         """
         Connect to a database created by :func:`gffutils.create_db`.
 
@@ -73,6 +75,13 @@ class FeatureDB(object):
             Default is False, since this includes a sorting step that can get
             time-consuming for many features.
 
+        pragmas : dict
+            Dictionary of pragmas to use when connecting to the database.  See
+            http://www.sqlite.org/pragma.html for the full list of
+            possibilities, and constants.default_pragmas for the defaults.
+            These can be changed later using the :meth:`FeatureDB.set_pragmas`
+            method.
+
         .. note::
 
             `dbfn` can also be a subclass of :class:`_DBCreator`, useful for
@@ -84,6 +93,7 @@ class FeatureDB(object):
         # to a new, separate (and empty) db in memory, we can alternatively
         # pass in a sqlite connection instance to use its existing, in-memory
         # db.
+        from gffutils import create
         if isinstance(dbfn, create._DBCreator):
             self.conn = dbfn.conn
             self.dbfn = dbfn.dbfn
@@ -96,6 +106,8 @@ class FeatureDB(object):
             raise ValueError(
                 "cannot connect to memory db; please provide the connection")
         else:
+            if not os.path.exists(dbfn):
+                raise ValueError("Database file %s does not exist" % dbfn)
             self.dbfn = dbfn
             self.conn = sqlite3.connect(self.dbfn)
 
@@ -105,6 +117,7 @@ class FeatureDB(object):
 
         self.default_encoding = default_encoding
         self.keep_order = keep_order
+        self.sort_attribute_values = sort_attribute_values
         c = self.conn.cursor()
 
         # Load some meta info
@@ -134,12 +147,34 @@ class FeatureDB(object):
             ''')
         self._autoincrements = dict(c)
 
+        self.set_pragmas(pragmas)
+
+    def set_pragmas(self, pragmas):
+        """
+        Set pragmas for the current database connection.
+
+        Parameters
+        ----------
+        pragmas : dict
+            Dictionary of pragmas; see constants.default_pragmas for a template
+            and http://www.sqlite.org/pragma.html for a full list.
+        """
+        self.pragmas = pragmas
+        c = self.conn.cursor()
+        c.executescript(
+            ';\n'.join(
+                ['PRAGMA %s=%s' % i for i in self.pragmas.items()]
+            )
+        )
+        self.conn.commit()
+
     def _feature_returner(self, **kwargs):
         """
         Returns a feature, adding additional database-specific defaults
         """
         kwargs.setdefault('dialect', self.dialect)
         kwargs.setdefault('keep_order', self.keep_order)
+        kwargs.setdefault('sort_attribute_values', self.sort_attribute_values)
         return Feature(**kwargs)
 
     def schema(self):
@@ -170,7 +205,7 @@ class FeatureDB(object):
         results = c.fetchone()
         # TODO: raise error if more than one key is found
         if results is None:
-            raise helpers.FeatureNotFoundError(key)
+            raise FeatureNotFoundError(key)
         return self._feature_returner(**results)
 
     def count_features_of_type(self, featuretype=None):
@@ -410,7 +445,7 @@ class FeatureDB(object):
             within `region`.
         """
         strand = None
-        if isinstance(region, basestring):
+        if isinstance(region, six.string_types):
             toks = region.split(':')
             seqid, coords = toks[:2]
             if len(toks) == 3:
@@ -449,7 +484,7 @@ class FeatureDB(object):
 
         # Add the featuretype clause
         if featuretype is not None:
-            if isinstance(featuretype, basestring):
+            if isinstance(featuretype, six.string_types):
                 featuretype = [featuretype]
             feature_clause = ' or '.join(
                 ['featuretype = ?' for _ in featuretype])
@@ -568,8 +603,9 @@ class FeatureDB(object):
 
         Remaining kwargs are passed to create_db.
         """
+        from gffutils import create
         if make_backup:
-            if isinstance(self.dbfn, basestring):
+            if isinstance(self.dbfn, six.string_types):
                 shutil.copy2(self.dbfn, self.dbfn + '.bak')
 
         # No matter what `features` came in as, convert to gffutils.Feature
@@ -577,7 +613,7 @@ class FeatureDB(object):
         # parsed into dicts in a Feature, we no longer have to worry about
         # that.  This also allows GTF features to be used to update a GFF
         # database, or vice versa.
-        if isinstance(features, basestring):
+        if isinstance(features, six.string_types):
             indb = create.create_db(features, intermediate, **kwargs)
             features = indb.all_features()
 
@@ -603,7 +639,8 @@ class FeatureDB(object):
         db._update_relations()
         db._finalize()
 
-    def add_relation(self, parent, child, level, parent_func=None, child_func=None):
+    def add_relation(self, parent, child, level, parent_func=None,
+                     child_func=None):
         """
         Manually add relations to the database.
 
@@ -635,9 +672,9 @@ class FeatureDB(object):
                 def child_func(parent, child):
                     child.attributes['Parent'] = parent['gene_id']
         """
-        if isinstance(parent, basestring):
+        if isinstance(parent, six.string_types):
             parent = self[parent]
-        if isinstance(child, basestring):
+        if isinstance(child, six.string_types):
             child = self[child]
 
         c = self.conn.cursor()
@@ -654,7 +691,6 @@ class FeatureDB(object):
 
         self.conn.commit()
 
-
     def _update(self, feature, cursor):
         values = [list(feature.astuple()) + [feature.id]]
         cursor.execute(
@@ -670,7 +706,6 @@ class FeatureDB(object):
             cursor.execute(
                 constants._INSERT, feature.astuple(self.default_encoding))
 
-
     def create_introns(self, exon_featuretype='exon',
                        grandparent_featuretype='gene', parent_featuretype=None,
                        new_featuretype='intron', merge_attributes=True):
@@ -737,6 +772,9 @@ class FeatureDB(object):
         """
         Merge overlapping features together.
 
+        Parameters
+        ----------
+
         features : iterator of Feature instances
 
         ignore_strand : bool
@@ -822,9 +860,22 @@ class FeatureDB(object):
 
     def children_bp(self, feature, child_featuretype='exon', merge=False):
         """
-        Returns the total bp of all children of a featuretype
+        Returns the total bp of all children of a featuretype.
 
         Useful for getting the exonic bp of an mRNA.
+
+        Parameters
+        ----------
+
+        feature : str or Feature instance
+
+        child_featuretype : str
+            Which featuretype to consider.  For example, to get exonic bp of an
+            mRNA, use `child_featuretype='exon'`.
+
+        merge : bool
+            Whether or not to merge child features together before summing
+            them.
         """
         children = self.children(feature, featuretype=child_featuretype,
                                  order_by='start')
@@ -836,6 +887,153 @@ class FeatureDB(object):
             total += len(child)
         return total
 
+    def bed12(self, feature, block_featuretype=['exon'],
+              thick_featuretype=['CDS'], thin_featuretype=None,
+              name_field='ID', color=None):
+        """
+        Converts `feature` into a BED12 format.
+
+        GFF and GTF files do not necessarily define genes consistently, so this
+        method provides flexible methods.
+
+        Parameters
+        ----------
+        feature : str or Feature instance
+            In most cases, this feature should be a transcript rather than
+            a gene.
+
+        block_featuretype : str or list
+            Which featuretype to use as the exons. These are represented as
+            blocks in the BED12 format.  Typically 'exon'.
+
+            Use the `thick_featuretype` and `thin_featuretype` arguments to
+            control the display of CDS as thicker blocks and UTRs as thinner
+            blocks.
+
+            Note that the features for `thick` or `thin` are *not*
+            automatically included in the blocks; if you do want them included,
+            then those featuretypes should be added to this `block_features`
+            list.
+
+            If no child features of type `block_featuretype` are found, then
+            the full `feature` is returned in BED12 format as if it had
+            a single exon.
+
+        thick_featuretype : str or list
+            Child featuretype(s) to use in order to determine the boundaries of
+            the "thick" blocks. In BED12 format, these represent coding
+            sequences; typically this would be set to "CDS".  This argument is
+            mutually exclusive with `thin_featuretype`.
+
+            Specifically, the BED12 thickStart will be the start coord of the
+            first `thick` item and the thickEnd will be the stop coord of the
+            last `thick` item.
+
+        thin_featuretype : str or list
+            Child featuretype(s) to use in order to determine the boundaries of
+            the "thin" blocks.  In BED12 format, these represent untranslated
+            regions.  Typically "utr" or ['three_prime_UTR', 'five_prime_UTR'].
+            Mutually exclusive with `thick_featuretype`.
+
+            Specifically, the BED12 thickStart field will be the stop coord of
+            the first `thin` item and the thickEnd field will be the start
+            coord of the last `thin` item.
+
+        name_field : str
+            Which attribute of `feature` to use as the feature's name.  If this
+            field is not present, a "." placeholder will be used instead.
+
+        color : None or str
+            If None, then use black (0,0,0) as the RGB color; otherwise this
+            should be a comma-separated string of R,G,B values each of which
+            are integers in the range 0-255.
+        """
+        if thick_featuretype and thin_featuretype:
+            raise ValueError("Can only specify one of `thick_featuertype` or "
+                             "`thin_featuretype`")
+
+        exons = list(self.children(feature, featuretype=block_featuretype,
+                                   order_by='start'))
+        if len(exons) == 0:
+            exons = [feature]
+        feature = self[feature]
+        first = exons[0].start
+        last = exons[-1].stop
+
+        if first != feature.start:
+            raise ValueError(
+                "Start of first exon (%s) does not match start of feature (%s)"
+                % (first, feature.start))
+        if last != feature.stop:
+            raise ValueError(
+                "End of last exon (%s) does not match end of feature (%s)"
+                % (last, feature.stop))
+
+        if color is None:
+            color = '0,0,0'
+        color = color.replace(' ', '').strip()
+
+        # Use field names as defined at
+        # http://genome.ucsc.edu/FAQ/FAQformat.html#format1
+        chrom = feature.chrom
+        chromStart = feature.start - 1
+        chromEnd = feature.stop
+        orig = constants.always_return_list
+        constants.always_return_list = True
+        try:
+            name = feature[name_field][0]
+        except KeyError:
+            name = "."
+
+        constants.always_return_list = orig
+
+        score = feature.score
+        if score == '.':
+            score = '0'
+        strand = feature.strand
+        itemRgb = color
+        blockCount = len(exons)
+        blockSizes = [len(i) for i in exons]
+        blockStarts = [i.start - 1 - chromStart for i in exons]
+
+        if thick_featuretype:
+            thick = list(self.children(feature, featuretype=thick_featuretype,
+                                       order_by='start'))
+            if len(thick) == 0:
+                thickStart = feature.start
+                thickEnd = feature.stop
+            else:
+                thickStart = thick[0].start - 1  # BED 0-based coords
+                thickEnd = thick[-1].stop
+
+        if thin_featuretype:
+            thin = list(self.children(feature, featuretype=thin_featuretype,
+                                      order_by='start'))
+            if len(thin) == 0:
+                thickStart = feature.start
+                thickEnd = feature.stop
+            else:
+                thickStart = thin[0].stop
+                thickEnd = thin[-1].start - 1  # BED 0-based coords
+
+        tst = chromStart + blockStarts[-1] + blockSizes[-1]
+        assert tst == chromEnd, "tst=%s; chromEnd=%s" % (tst, chromEnd)
+
+        fields = [
+            chrom,
+            chromStart,
+            chromEnd,
+            name,
+            score,
+            strand,
+            thickStart,
+            thickEnd,
+            itemRgb,
+            blockCount,
+            ','.join(map(str, blockSizes)),
+            ','.join(map(str, blockStarts))]
+        return '\t'.join(map(str, fields))
+
     # Recycle the docs for _relation so they stay consistent between parents()
     # and children()
     children.__doc__ = children.__doc__.format(
