@@ -1,21 +1,22 @@
 import warnings
-import expected
+from . import expected
 from gffutils import example_filename, create, parser, feature
 import gffutils
 import gffutils.helpers as helpers
 import gffutils.gffwriter as gffwriter
 import sys
 import os
+import six
 import shutil
-import sqlite3
-import nose.tools as nt
-import difflib
-import pprint
-import copy
+import threading
 import tempfile
 from textwrap import dedent
 from nose.tools import assert_raises
-
+from six.moves import SimpleHTTPServer
+if sys.version_info.major == 3:
+    import socketserver as SocketServer
+else:
+    import SocketServer
 
 testdbfn_gtf = ':memory:'
 testdbfn_gff = ':memory:'
@@ -24,6 +25,7 @@ def test_update():
     # check both in-memory and file-based dbs
     db = create.create_db(
         example_filename('FBgn0031208.gff'), ':memory:', verbose=False,
+        keep_order=True,
         force=True)
 
     orig_num_features = len(list(db.all_features()))
@@ -36,7 +38,9 @@ def test_update():
     db.update([f])
     x = list(db.features_of_type('testing'))
     assert len(x) == 1
-    assert str(x[0]) == "chr2L	.	testing	1	10	.	+	.	ID=testing_feature;n=1", str(x[0])
+    x = x[0]
+    x.keep_order = True
+    assert str(x) == "chr2L	.	testing	1	10	.	+	.	ID=testing_feature;n=1", str(x)
 
     # ought to be one more now . . .
     num_features = len(list(db.all_features()))
@@ -47,11 +51,18 @@ def test_update():
     f = feature.feature_from_line(
         'chr2L . testing 1 10 . + . ID=testing_feature;n=1',
         dialect=db.dialect, strict=False)
+    f.keep_order = True
     f.attributes['n'] = ['2']
     db.update([f], merge_strategy='merge')
     x = list(db.features_of_type('testing'))
     assert len(x) == 1
-    assert str(x[0]) == "chr2L	.	testing	1	10	.	+	.	ID=testing_feature;n=1,2"
+
+    # Merging does a list(set()) operation, so the order is not guaranteed.
+    # Fix it here for testing...
+    x = x[0]
+    x.attributes['n'].sort()
+
+    assert str(x) == "chr2L	.	testing	1	10	.	+	.	ID=testing_feature;n=1,2", str(x)
 
     # still should have the same number of features as before (still 2)
     num_features = len(list(db.all_features()))
@@ -61,7 +72,7 @@ def test_update():
     # Merging while iterating.  e.g., if you're updating children with gene
     # IDs.
     db = create.create_db(example_filename('FBgn0031208.gff'), ':memory:',
-                          verbose=False, force=True)
+                          verbose=False, force=True, keep_order=True)
     for gene in db.features_of_type('gene'):
         for child in list(db.children(gene)):
             # important: the FBgn0031208.gff file was designed to have some
@@ -74,9 +85,9 @@ def test_update():
                 child.attributes['ID'] = [child.id]
             db.update([child], merge_strategy='replace')
 
-    print "\n\nafter\n\n"
+    print("\n\nafter\n\n")
     for child in db.children(gene):
-        print child.id
+        print(child.id)
         assert child.attributes['gene_id'] == ['FBgn0031208'], (child, child.attributes)
 
     num_entries = 0
@@ -86,7 +97,7 @@ def test_update():
         rec.attributes["new"] = ["new_value"]
         db.update([rec])
         num_entries += 1
-    print list(db.all_features())
+    print(list(db.all_features()))
 
 
     assert (num_entries > 1), "Only %d left after update" % (num_entries)
@@ -96,11 +107,14 @@ def test_update():
     f = feature.feature_from_line(
         'chr2L . testing 1 10 . + . ID=testing_feature;n=1',
         dialect=db.dialect, strict=False)
+
+    f.keep_order = True
+
     f.attributes['n'] = ['3']
     db.update([f], merge_strategy='replace')
     x = list(db.features_of_type('testing'))
     assert len(x) == 1
-    assert str(x[0]) == "chr2L	.	testing	1	10	.	+	.	ID=testing_feature;n=3"
+    assert str(x[0]) == "chr2L	.	testing	1	10	.	+	.	ID=testing_feature;n=3", str(x[0])
     # still should have the same number of features as before (still 2)
     num_features = len(list(db.all_features()))
     assert num_features == orig_num_features + 1, num_features
@@ -109,12 +123,14 @@ def test_update():
     # Same thing, but GTF instead of GFF.
     db = create.create_db(
         example_filename('FBgn0031208.gtf'), ':memory:', verbose=False,
-        force=True)
+        force=True, keep_order=True)
     f = feature.feature_from_line('chr2L . testing 1 10 . + . gene_id "fake"; n "1"', strict=False)
+    f.keep_order = True
     db.update([f], merge_strategy='merge')
     x = list(db.features_of_type('testing'))
     assert len(x) == 1
     x = x[0]
+    x.keep_order = True
 
     # note the trailing semicolon.  That's because the db's dialect has
     # ['trailing semicolon'] = True.
@@ -154,7 +170,8 @@ class BaseDB(object):
             ':memory:',
             id_spec=id_func,
             merge_strategy='create_unique',
-            verbose=False
+            verbose=False,
+            keep_order=True
         )
         self.c = self.db.conn.cursor()
         self.dialect = self.db.dialect
@@ -169,7 +186,7 @@ class BaseDB(object):
         """Count using SQL"""
         self.c.execute('select count() from features where featuretype = ?',(featuretype,))
         results = self.c.fetchone()[0]
-        print 'count1("%s") says: %s' % (featuretype,results)
+        print('count1("%s") says: %s' % (featuretype,results))
         return results
 
     def _count2(self,featuretype):
@@ -185,13 +202,13 @@ class BaseDB(object):
 
             if L[2] == featuretype:
                 cnt += 1
-        print 'count2("%s") says: %s' % (featuretype, cnt)
+        print('count2("%s") says: %s' % (featuretype, cnt))
         return cnt
 
     def _count3(self,featuretype):
         """Count with the count_features_of_type method"""
         results = self.db.count_features_of_type(featuretype)
-        print 'count3("%s") says: %s' % (featuretype, results)
+        print('count3("%s") says: %s' % (featuretype, results))
         return results
 
     def _count4(self,featuretype):
@@ -199,20 +216,20 @@ class BaseDB(object):
         cnt = 0
         for i in self.db.features_of_type(featuretype):
             cnt += 1
-        print 'count4("%s") says: %s' % (featuretype,cnt)
+        print('count4("%s") says: %s' % (featuretype,cnt))
         return cnt
 
     def featurecount_test(self):
         #  Right number of each featuretype, using multiple different ways of
         #  counting?
-        print 'format:', self.dialect['fmt']
+        print('format:', self.dialect['fmt'])
         expected_feature_counts = expected.expected_feature_counts[self.dialect['fmt']]
         for featuretype, expected_count in expected_feature_counts.items():
             rawsql_cnt = self._count1(featuretype)
             fileparsed_cnt = self._count2(featuretype)
             count_feature_of_type_cnt = self._count3(featuretype)
             iterator_cnt = self._count4(featuretype)
-            print "expected count:", expected_count
+            print("expected count:", expected_count)
             assert rawsql_cnt == count_feature_of_type_cnt == iterator_cnt == fileparsed_cnt == expected_count
 
     def _expected_parents(self):
@@ -228,8 +245,8 @@ class BaseDB(object):
         parents1, parents2 = self._expected_parents()
         for child, expected_parents in parents1.items():
             observed_parents = [i.id for i in self.db.parents(child, level=1)]
-            print 'observed parents for %s:' % child, set(observed_parents)
-            print 'expected parents for %s:' % child, set(expected_parents)
+            print('observed parents for %s:' % child, set(observed_parents))
+            print('expected parents for %s:' % child, set(expected_parents))
             assert set(observed_parents) == set(expected_parents)
 
 
@@ -237,12 +254,27 @@ class BaseDB(object):
         parents1, parents2 = self._expected_parents()
         for child, expected_parents in parents2.items():
             observed_parents = [i.id for i in self.db.parents(child, level=2)]
-            print self.db[child]
-            print 'observed parents for %s:' % child, set(observed_parents)
-            print 'expected parents for %s:' % child, set(expected_parents)
+            print(self.db[child])
+            print('observed parents for %s:' % child, set(observed_parents))
+            print('expected parents for %s:' % child, set(expected_parents))
             assert set(observed_parents) == set(expected_parents)
 
 
+    def test_bed12(self):
+        if self.__class__ == TestGFFClass:
+            kwargs = dict(block_featuretype='exon', thick_featuretype='CDS', name_field='ID')
+        if self.__class__ == TestGTFClass:
+            kwargs = dict(block_featuretype='exon', thick_featuretype='CDS', name_field='transcript_id')
+        obs = self.db.bed12('FBtr0300689', **kwargs)
+        exp = "chr2L	7528	9484	FBtr0300689	0	+	7679	8610	0,0,0	2	588,1292	0,664"
+        assert obs == exp
+
+
+        obs = self.db.bed12('FBtr0300690', **kwargs)
+        exp = "chr2L	7528	9484	FBtr0300690	0	+	7679	9276	0,0,0	3	588,397,817	0,664,1139"
+        assert obs == exp
+
+
 class TestGFFClass(BaseDB):
     orig_fn = example_filename('FBgn0031208.gff')
 
@@ -264,14 +296,14 @@ def test_random_chr():
         assert (mRNA_entry.featuretype == "mRNA"), \
                "Not all entries are of type mRNA! %s" \
                %(",".join([entry.featuretype for entry in mRNAs]))
-    print "Parsed random chromosome successfully."
+    print("Parsed random chromosome successfully.")
 
 
 def test_gffwriter():
     """
     Test GFFWriter.
     """
-    print "Testing GFF writer.."
+    print("Testing GFF writer..")
     fn = gffutils.example_filename("unsanitized.gff")
     # Make a copy of it as temporary named file
     temp_f = tempfile.NamedTemporaryFile(delete=False)
@@ -281,13 +313,13 @@ def test_gffwriter():
     source_first_line = open(temp_fname_source, "r").readline().strip()
     assert (not source_first_line.startswith("#GFF3")), \
            "unsanitized.gff should not have a gffutils-style header."
-    db_in = gffutils.create_db(fn, ":memory:")
+    db_in = gffutils.create_db(fn, ":memory:", keep_order=True)
     # Fetch first record
-    rec = db_in.all_features().next()
+    rec = next(db_in.all_features())
     ##
     ## Write GFF file in-place test
     ##
-    print "Testing in-place writing"
+    print("Testing in-place writing")
     gff_out = gffwriter.GFFWriter(temp_fname_source,
                                   in_place=True,
                                   with_header=True)
@@ -298,11 +330,11 @@ def test_gffwriter():
     new_header = rewritten.readline().strip()
     assert new_header.startswith("#GFF3"), \
            "GFFWriter serialized files should have a #GFF3 header."
-    print "  - Wrote GFF file in-place successfully."
+    print("  - Wrote GFF file in-place successfully.")
     ##
     ## Write GFF file to new file test
     ##
-    print "Testing writing to new file"
+    print("Testing writing to new file")
     new_file = tempfile.NamedTemporaryFile(delete=False)
     gff_out = gffwriter.GFFWriter(new_file.name)
     gff_out.write_rec(rec)
@@ -310,7 +342,7 @@ def test_gffwriter():
     new_line = open(new_file.name, "r").readline().strip()
     assert new_line.startswith("#GFF3"), \
            "GFFWriter could not write to a new GFF file."
-    print "  - Wrote to new file successfully."
+    print("  - Wrote to new file successfully.")
 
 
 
@@ -326,8 +358,8 @@ def test_gffwriter():
 #                             force=True)
 #     gene_id = "FBgn0031208"
 #     gene_childs = list(db.children(gene_id))
-#     print "First child is not an mRNA"
-#     print gene_childs[0].featuretype
+#     print("First child is not an mRNA")
+#     print(gene_childs[0].featuretype)
 #     assert str(gene_childs[0].attributes) == 'ID=FBtr0300689;Name=CG11023-RB;Parent=FBgn0031208;Dbxref=FlyBase_Annotation_IDs:CG11023-RB;score_text=Strongly Supported;score=11'
 #     gene_childs[0].attributes["ID"] = "Modified"
 #     assert str(gene_childs[0].attributes) == 'ID=Modified;Name=CG11023-RB;Parent=FBgn0031208;Dbxref=FlyBase_Annotation_IDs:CG11023-RB;score_text=Strongly Supported;score=11;ID=Modified'
@@ -342,14 +374,14 @@ def test_create_db_from_iter():
     """
     Test creation of FeatureDB from iterator.
     """
-    print "Testing creation of DB from iterator"
+    print("Testing creation of DB from iterator")
     db_fname = gffutils.example_filename("gff_example1.gff3")
-    db = gffutils.create_db(db_fname, ":memory:")
+    db = gffutils.create_db(db_fname, ":memory:", keep_order=True)
     def my_iterator():
         for rec in db.all_features():
             yield rec
-    new_db = gffutils.create_db(my_iterator(), ":memory:")
-    print list(new_db.all_features())
+    new_db = gffutils.create_db(my_iterator(), ":memory:", keep_order=True)
+    print(list(new_db.all_features()))
     gene_feats = new_db.all_features(featuretype="gene")
     assert (len(list(gene_feats)) != 0), "Could not load genes from GFF."
 
@@ -369,12 +401,12 @@ def test_sanitize_gff():
     # starts must be less than or equal to stops
     for rec in sanitized_recs.all_features():
         assert (rec.start <= rec.stop), "Sanitization failed."
-    print "Sanitized GFF successfully."
+    print("Sanitized GFF successfully.")
 
 
 def test_region():
     db_fname = gffutils.example_filename("gff_example1.gff3")
-    db = gffutils.create_db(db_fname, ":memory:")
+    db = gffutils.create_db(db_fname, ":memory:", keep_order=True)
     all_in_region = list(db.region("chr1:4000000-5000000"))
     all_minus = list(db.region("chr1:4000000-5000000:-"))
     all_plus = list(db.region("chr1:4000000-5000000:+"))
@@ -392,16 +424,17 @@ def test_region():
 def test_nonascii():
     # smoke test (prev. version returned Unicode)
     #
-    db = gffutils.create_db(gffutils.example_filename('nonascii'), ":memory:")
+    db = gffutils.create_db(gffutils.example_filename('nonascii'), ":memory:",
+                            keep_order=True)
     for i in db.all_features():
         # this works in IPython, or using nosetests --with-doctest...
         try:
-            print i
+            print(i)
 
         # ...but fails using plain nosetests or when using regular Python
         # interpreter
         except UnicodeEncodeError:
-            print unicode(i)
+            print(six.text_type(i))
 
 
 def test_feature_merge():
@@ -411,53 +444,73 @@ def test_feature_merge():
     chr1	a	testing	1	10	.	+	.	gene_id "fake"; n "2";
     chr1	b	testing	1	10	.	+	.	gene_id "fake"; n "1";
     """)
-    db = gffutils.create_db(gtfdata, ":memory:", from_string=True, merge_strategy='merge', id_spec='gene_id', force_merge_fields=['source'])
+    db = gffutils.create_db(gtfdata, ":memory:", from_string=True,
+                            merge_strategy='merge', id_spec='gene_id',
+                            force_merge_fields=['source'], keep_order=True,
+                            sort_attribute_values=True)
     assert db.dialect['fmt'] == 'gtf'
     assert len(list(db.all_features())) == 1
     x = db['fake']
+    x.keep_order = True
+    x.attributes['n'].sort()
     assert str(x) == 'chr1	a,b	testing	1	10	.	+	.	gene_id "fake"; n "1,2";', str(x)
 
     gffdata = dedent("""
     chr1	a	testing	1	10	.	+	.	gene_id="fake"; n="2";
     chr1	b	testing	1	10	.	+	.	gene_id="fake"; n="1";
     """)
-    db = gffutils.create_db(gffdata, ":memory:", from_string=True, merge_strategy='merge', id_spec='gene_id', force_merge_fields=['source'])
+    db = gffutils.create_db(gffdata, ":memory:", from_string=True,
+                            merge_strategy='merge', id_spec='gene_id',
+                            force_merge_fields=['source'], keep_order=True)
     assert db.dialect['fmt'] == 'gff3'
     assert len(list(db.all_features())) == 1
     x = db['fake']
+    x.attributes['n'].sort()
+    x.keep_order = True
     assert str(x) == 'chr1	a,b	testing	1	10	.	+	.	gene_id="fake"; n="1,2";', str(x)
 
 
     # But when not using force_merge_fields, there should be separate entries;
     # accessing fake and fake_1 should not give FeatureNotFound errors.
-    db = gffutils.create_db(gtfdata, ':memory:', from_string=True, merge_strategy='merge', id_spec='gene_id')
+    db = gffutils.create_db(gtfdata, ':memory:', from_string=True,
+                            merge_strategy='merge', id_spec='gene_id',
+                            keep_order=True)
     assert len(list(db.all_features())) == 2
     x = db['fake']
     y = db['fake_1']
 
-    db = gffutils.create_db(gffdata, ':memory:', from_string=True, merge_strategy='merge', id_spec='gene_id')
+    db = gffutils.create_db(gffdata, ':memory:', from_string=True,
+                            merge_strategy='merge', id_spec='gene_id',
+                            keep_order=True)
     assert len(list(db.all_features())) == 2
     x = db['fake']
     y = db['fake_1']
 
 
-    assert_raises(ValueError, gffutils.create_db, gtfdata, ":memory:", from_string=True, merge_strategy='merge', id_spec='gene_id', force_merge_fields=['start'])
+    assert_raises(ValueError, gffutils.create_db, gtfdata, ":memory:",
+                  from_string=True, merge_strategy='merge', id_spec='gene_id',
+                  force_merge_fields=['start'], keep_order=True)
 
     # test that warnings are raised because of strand and frame
-    with warnings.catch_warnings(True) as w:
+    with warnings.catch_warnings(record=True) as w:
         gffdata = dedent("""
         chr1	a	testing	1	10	.	+	.	gene_id="fake"; n="2";
         chr1	a	testing	1	10	.	-	1	gene_id="fake"; n="1";
         """)
-        db = gffutils.create_db(gffdata, ":memory:", from_string=True, merge_strategy='merge', id_spec='gene_id', force_merge_fields=['strand', 'frame'])
+        db = gffutils.create_db(gffdata, ":memory:", from_string=True,
+                                merge_strategy='merge', id_spec='gene_id',
+                                force_merge_fields=['strand', 'frame'],
+                                keep_order=True, sort_attribute_values=True)
         assert db.dialect['fmt'] == 'gff3'
         assert len(list(db.all_features())) == 1
         x = db['fake']
-        assert str(x) == 'chr1	a	testing	1	10	.	+,-	1,.	gene_id="fake"; n="1,2";', str(x)
+        x.keep_order = True
+        x.attributes['n'].sort()
+        assert str(x) == 'chr1	a	testing	1	10	.	+,-	.,1	gene_id="fake"; n="1,2";', str(x)
         assert len(w) == 2
 
 def test_add_relation():
-    db = gffutils.create_db(gffutils.example_filename('FBgn0031208.gff'), ':memory:')
+    db = gffutils.create_db(gffutils.example_filename('FBgn0031208.gff'), ':memory:', keep_order=True)
     L = len(list(db.children('FBgn0031208:3')))
     assert L == 0, L
 
@@ -477,6 +530,48 @@ def test_add_relation():
     assert x['exon_parent'] == ['FBgn0031208:3']
 
 
+def test_create_db_from_url():
+    """
+    Test creation of FeatureDB from URL iterator.
+    """
+    print("Testing creation of DB from URL iterator")
+    # initially run SimpleHTTPServer at port 0 and os will take first available
+    Handler = SimpleHTTPServer.SimpleHTTPRequestHandler
+    httpd = SocketServer.TCPServer(("", 0), Handler)
+    port = str(httpd.socket.getsockname()[1])
+    print("serving at port", port)
+
+    # Serving test/data folder
+    served_folder = gffutils.example_filename('')
+    os.chdir(served_folder)
+
+    print("Starting SimpleHTTPServer in thread")
+    server_thread = threading.Thread(target=httpd.serve_forever)
+    server_thread.deamon = True
+    server_thread.start()
+    try:
+        url = ''.join(['http://localhost:', port, '/gff_example1.gff3'])
+        db = gffutils.create_db(url, ":memory:", keep_order=True)
+        def my_iterator():
+            for rec in db.all_features():
+                yield rec
+        new_db = gffutils.create_db(my_iterator(), ":memory:", keep_order=True)
+
+        print(list(new_db.all_features()))
+        gene_feats = new_db.all_features(featuretype="gene")
+        assert (len(list(gene_feats)) != 0), "Could not load genes from GFF."
+    finally:
+        print('Server shutdown.')
+        httpd.shutdown()
+        server_thread.join()
+
+
+def test_empty_files():
+    fn = tempfile.NamedTemporaryFile(delete=False).name
+    a = open(fn, 'w')
+    a.close()
+    assert_raises(ValueError, gffutils.create_db, fn, fn + '.db')
+
 
 if __name__ == "__main__":
     # this test case fails
